@@ -3,7 +3,6 @@
 
 import os.path
 import numpy as np
-# from scipy.stats import trimboth
 from loguru import logger
 
 from kivy.app import App
@@ -16,16 +15,18 @@ from kivy.core.window import Window
 from jocular.table import Table
 from jocular.utils import make_unique_filename, toast, percentile_clip
 from jocular.component import Component
-from jocular.settingsmanager import Settings
+from jocular.settingsmanager import JSettings
 from jocular.image import Image, save_image, fits_in_dir
 from jocular.exposurechooser import exp_to_str
 from jocular.gradient import estimate_background
 
 date_time_format = '%d %b %y %H:%M'
 
+
 def none_to_empty(x):
     return '' if x is None else x
 
+
 def subregion(m, s):
     ''' Return x and y limits of master (m) that correspond 
         to sub (s), or None if not possible to use this master
@@ -34,6 +35,14 @@ def subregion(m, s):
     if m is None:
         return None
 
+    ''' if camera is unknown or different for master and sub,
+        only allow subregion processing if the shape is identical
+    '''
+
+    if (m.camera is None) or (s.camera is None) or (m.camera != s.camera):
+        if (m.shape[0] != s.shape[0]) or (m.shape[1] != s.shape[1]):
+            return None
+
     # extract region covered by m and s
     mx = 0 if m.ROI_x is None else m.ROI_x
     my = 0 if m.ROI_y is None else m.ROI_y
@@ -56,14 +65,14 @@ def subregion(m, s):
 
     return None
 
-class Calibrator(Component, Settings):
+
+class Calibrator(Component, JSettings):
 
     save_settings = ['apply_dark', 'apply_flat']
 
     masters = DictProperty({})
     apply_flat = BooleanProperty(False)
     apply_dark = BooleanProperty(False)
-
     use_l_filter = BooleanProperty(True)
     remove_hot_pixels = BooleanProperty(True)
     fd_exposure_tol = NumericProperty(.1)
@@ -133,16 +142,17 @@ class Calibrator(Component, Settings):
                 if s.is_master:
                     self.add_to_library(s)
             except Exception as e:
-                logger.warning('Calibrator: unable to parse calibration {:} ({:})'.format(f, e))
+                logger.warning(f'Calibrator: unable to parse calibration {f} ({e})')
 
 
     def on_new_object(self, *args):
         n_masters = len(self.library)
         if n_masters > 0:
-            self.info('{:d} masters'.format(n_masters))
+            self.info(f'{n_masters:d} masters')
         else:
             self.info('no masters')
 
+
     def add_to_library(self, m):
         ''' called on initialisation and when we save a new master
         '''
@@ -159,7 +169,6 @@ class Calibrator(Component, Settings):
             'offset': none_to_empty(m.offset),
             'bin': none_to_empty(m.binning),
             'calibration_method': none_to_empty(m.calibration_method),
-            #'ROI': none_to_empty(m.ROI),
             'filter': m.filter,
             'created': m.create_time.strftime(date_time_format),
             'shape_str': m.shape_str,
@@ -192,9 +201,17 @@ class Calibrator(Component, Settings):
 
         ''' Remove hot pixels from calibration frame
         '''
-        master = Component.get('BadPixelMap').remove_hotpix(
-            master, 
-            apply_BPM=self.remove_hot_pixels)
+
+        # possibly shouldn't do this for dark/bias frames
+        ''' to do: figure out how best to handle bad pixel mapping for
+            calibration frames now that I'm not saving BPM from
+            one capture to the next
+        '''
+        # if sub_type == 'flat':
+        #     master = Component.get('BadPixelMap').remove_hotpix(
+        #         master, 
+        #         apply_BPM=self.remove_hot_pixels)
+
 
         ''' Flats were divided thru by their robust mean to account for 
             level differences but then scaled to 50% to enable B/W controls; 
@@ -218,19 +235,19 @@ class Calibrator(Component, Settings):
             calibration masters.
         '''
 
-        name = 'master{:}.fit'.format(capture_props['sub_type'])
+        name = f'master{capture_props["sub_type"]}.fit'
         path = make_unique_filename(os.path.join(self.calibration_dir, name))
         save_image(data=data, path=path, capture_props=capture_props)
         self.add_to_library(Image(path))
 
         # add to notes field of current DSO
-        notes = 'Exposure {:}\n'.format(exp_to_str(capture_props.get('exposure', 0)))
-        notes += '\n'.join(['{:} {:}'.format(k, v) for k, v in capture_props.items() 
+        notes = f'Exposure {exp_to_str(capture_props.get("exposure", 0))}\n'
+        notes += '\n'.join([f'{k} {v}' for k, v in capture_props.items() 
             if k in {'filter', 'temperature', 'gain', 'offset', 'camera', 
                 'binning', 'calibration_method'}])
         Component.get('Notes').notes = notes
 
-        logger.info('new master {:}'.format(capture_props))
+        logger.info(f'new master {capture_props}')
 
 
     def calibrate(self, sub):
@@ -249,12 +266,14 @@ class Calibrator(Component, Settings):
             self.info('none')
             return
 
+        logger.trace('starting calibration')
+
         # get all masters anyway (~1ms)
         dark = self.get_dark(sub)
         flat = self.get_flat(sub)
         bias = self.get_bias(sub)
         
-        logger.debug('D {:} F {:} B {:}'.format(dark, flat, bias))
+        logger.trace(f'D {dark} F {flat} B {bias}')
 
         D = self.get_master(dark)
         F = self.get_master(flat)
@@ -265,15 +284,14 @@ class Calibrator(Component, Settings):
         # to apply dark we just need a dark
         if self.apply_dark and dark is not None:
             dx0, dx1, dy0, dy1 = subregion(self.masters[dark], sub)
-            logger.trace('Dark subregion {:} {:} {:} {:}'.format(
-                dx0, dx1, dy0, dy1))
+            logger.trace(f'Dark subregion {dx0} {dx1} {dy0} {dy1}')
             im = im - D[dy0: dy1, dx0: dx1]
             sub.calibrations = {'dark'}
 
         # to apply flat we need a flat and either a bias or a dark
         if self.apply_flat and flat is not None:
+            fx0, fx1, fy0, fy1 = subregion(self.masters[flat], sub)
             if 'dark' in sub.calibrations:
-                fx0, fx1, fy0, fy1 = subregion(self.masters[flat], sub)
                 im = im / F[fy0: fy1, fx0: fx1]
                 sub.calibrations = {'dark', 'flat'}
             elif bias is not None:
@@ -298,18 +316,15 @@ class Calibrator(Component, Settings):
         im[im > 1] = 1
 
         sub.image = im
+
+        logger.trace('calibration complete')
+
         applied = ' '.join(list(sub.calibrations))
         if applied:
             self.info(applied)
         else:
             self.info('none suitable')
 
-    ''' TO DO: enable ROI of masters
-        need a 'compatible' function that checks offsets
-        and also a function that returns offsets that can be used
-        during the calibration process itself 
-    '''
-
 
     def get_dark(self, sub, exposure_tol=None):
         ''' find darks with same camera, gain, offset, binning, shape, and
@@ -323,13 +338,33 @@ class Calibrator(Component, Settings):
         if exposure_tol is None:
             exposure_tol = self.exposure_tol
 
+        # diagnostics
+        if False:
+            for k, v in self.masters.items():
+                logger.trace('')
+                logger.trace(f'Testing {k}, {v}')
+                if v.sub_type != 'dark':
+                    logger.trace('not dark')
+                if v.binning != sub.binning:
+                    logger.trace(f'different binning {v.binning} vs {sub.binning}')
+                if subregion(v, sub) is None:
+                    logger.trace('incompatible subregion')
+                if v.gain != (sub.gain if v.gain is not None else v.gain):
+                    logger.trace(f'different gain {v.gain} vs {sub.gain}')
+                if v.offset != (sub.offset if v.offset is not None else v.offset):
+                    logger.trace(f'different offset {v.offset} vs {sub.offset}')
+                if v.camera != (sub.camera if v.camera is not None else v.camera):
+                    logger.trace(f'different camera {v.camera} vs {sub.camera}')
+                if v.exposure is not None and abs(v.exposure - sub.exposure) > self.exposure_tol:
+                    logger.trace(f'exposure outside tolerance {v.exposure} vs {sub.exposure}')
+
         darks = {k: v for k, v in self.masters.items()
                     if v.sub_type == 'dark' and
                         v.binning == sub.binning and
-                        subregion(v, sub) != None and
-                        v.gain == sub.gain and
-                        v.offset == sub.offset and
-                        v.camera == (sub.camera if sub.camera is not None else v.camera) and
+                        subregion(v, sub) is not None and
+                        v.gain == (sub.gain if v.gain is not None else v.gain) and
+                        v.offset == (sub.offset if v.offset is not None else v.offset) and
+                        # v.camera == (sub.camera if v.camera is not None else v.camera) and
                         v.exposure is not None and
                         abs(v.exposure - sub.exposure) <= self.exposure_tol
                 }
@@ -358,7 +393,7 @@ class Calibrator(Component, Settings):
 
         # find the one with the closest exposure
         for k in sorted(darks, key=darks.get):
-            logger.debug('matching flatdark {:}'.format(k))
+            logger.debug(f'matching flatdark {k}')
             return k
 
 
@@ -370,10 +405,10 @@ class Calibrator(Component, Settings):
         bias = {k: v.age for k, v in self.masters.items()
                     if v.sub_type == 'bias' and
                         v.binning == sub.binning and 
-                        subregion(v, sub) != None and
-                        v.gain == sub.gain and
-                        v.offset == sub.offset and
-                        v.camera == (sub.camera if sub.camera is not None else v.camera)
+                        subregion(v, sub) is not None and
+                        v.gain == (sub.gain if v.gain is not None else v.gain) and
+                        v.offset == (sub.offset if v.offset is not None else v.offset) and
+                        v.camera == (sub.camera if v.camera is not None else v.camera)
                 }
 
         return min(bias, key=bias.get) if len(bias) > 0 else None
@@ -386,9 +421,9 @@ class Calibrator(Component, Settings):
 
         flats = {k: v for k, v in self.masters.items()
                     if v.sub_type == 'flat' and
-                        v.binning == (sub.binning if sub.binning is not None else v.binning) and 
-                        subregion(v, sub) != None and
-                        v.camera == (sub.camera if sub.camera is not None else v.camera)
+                        v.binning == sub.binning and 
+                        subregion(v, sub) is not None and
+                        v.camera == (sub.camera if v.camera is not None else v.camera)
                 }
 
         # flat in required filter
@@ -397,7 +432,7 @@ class Calibrator(Component, Settings):
         else:
             flats_in_filt = {} 
 
-        # if we have none and can use L filter, use these
+        # if we have none and can use L or C filter, use these
         if (len(flats_in_filt) == 0) and self.use_l_filter:
             flats_in_filt = {k:v for k, v in flats.items() if v.filter == 'L'}
 
@@ -419,10 +454,12 @@ class Calibrator(Component, Settings):
         # Retrieve image (NB loaded on demand, so effectively a cache)
         return self.masters[name].get_image()
 
+
     def _most_subs(self, cands):
         c = {k: cands[k]['nsubs'] for k in cands.keys()}
         return max(c, key=c.get)
 
+
     def calibrate_flat(self, sub):
         ''' Perform calibrations on flat which include subtracting 
         bias/flat-dark/constant if possible, then rescaling so the 
@@ -470,7 +507,6 @@ class Calibrator(Component, Settings):
         sub.image = .5 * im / robust_mean
 
 
-    
     def build_calibrations(self):
         ''' Contruct table from library
         '''
@@ -481,27 +517,28 @@ class Calibrator(Component, Settings):
             name='Calibration masters',
             description='Calibration masters',
             cols={
-                'Name': {'w': 120, 'align': 'left', 'field': 'name', 
-                'action': self.show_calibration_frame},
-                'Camera': {'w': 140, 'align': 'left', 'field': 'camera', 'type': str},
-                'Type': {'w': 60, 'field': 'type', 'align': 'left'},
-                'Exposure': {'w': 80, 'field': 'exposure'},
-                'Temp. C': {'w': 80, 'field': 'temperature', 'type': str},
-                'Gain': {'w': 50, 'field': 'gain', 'type': int},
-                'Offset': {'w': 60, 'field': 'offset', 'type': int},
-                #'ROI': {'w': 80, 'field': 'ROI', 'type': str},
-                'Bin': {'w': 45, 'field': 'bin', 'type': int},
-                'Calib': {'w': 120, 'field': 'calibration_method', 'type': str},
-                'Filter': {'w': 80, 'field': 'filter'},
-                'Created': {'w': 120, 'field': 'created', 'sort': {'DateFormat': date_time_format}},
-                'Size': {'w': 110, 'field': 'shape_str'},
-                'Age': {'w': 50, 'field': 'age', 'type': int},
-                'Subs': {'w': 50, 'field': 'nsubs', 'type': int}
+                'name': {'w': 120, 'align': 'left', 'label': 'Name', 'action': self.show_calibration_frame},
+                'type': {'w': 60, 'label': 'Type', 'align': 'left'},
+                'exposure': {'w': 80, 'label': 'Exposure'},
+                'temperature': {'w': 80, 'label': 'Temp. C', 'type': str},
+                'gain': {'w': 50, 'label': 'Gain', 'type': int},
+                'offset': {'w': 60, 'label': 'Offset', 'type': int},
+                'bin': {'w': 45, 'label': 'Bin', 'type': int},
+                'calibration_method': {'w': 120, 'label': 'Calib?', 'type': str},
+                'filter': {'w': 80, 'label': 'Filter'},
+                'created': {'w': 130, 'label': 'Created', 'sort': {'DateFormat': date_time_format}},
+                'shape_str': {'w': 110, 'label': 'Size'},
+                'age': {'w': 50, 'label': 'Age', 'type': int},
+                'nsubs': {'w': 50, 'label': 'Subs', 'type': int},
+                'camera': {'w': 1, 'align': 'left', 'label': 'Camera', 'type': str},
                 },
             actions={'move to delete dir': self.move_to_delete_folder},
+            initial_sort_column='created',
+            reverse_initial_sort=True,
             on_hide_method=self.app.table_hiding
             )
 
+
     def show_calibration_table(self, *args):
         ''' Called when user clicks 'library' on GUI
         '''
@@ -516,7 +553,8 @@ class Calibrator(Component, Settings):
 
         self.calibration_table.show()    
 
-    def show_calibration_frame(self, row):
+
+    def show_calibration_frame(self, row, col, value):
         self.calibration_table.hide()
         # convert row.key to str (it is numpy.str by default)
         # get image from path
@@ -527,7 +565,7 @@ class Calibrator(Component, Settings):
                 im.image /= 2
             Component.get('Monochrome').display_sub(im.image)
         except Exception as e:
-            logger.error('{:}'.format(e))
+            logger.error(f'{e}')
 
 
     def move_to_delete_folder(self, *args):
@@ -537,5 +575,5 @@ class Calibrator(Component, Settings):
                 objio.delete_file(os.path.join(self.calibration_dir, nm))
                 del self.library[nm]
                 del self.masters[nm]
-        logger.info('deleted {:} calibration masters'.format(len(self.calibration_table.selected)))
+        logger.info(f'deleted {len(self.calibration_table.selected)} calibration masters')
         self.calibration_table.update()
