@@ -1,6 +1,8 @@
 from jocular.device import Device
 from kivy.properties import StringProperty
+#from jocular.settingsmanager import JSettings
 
+#class GenericCamera(JSettings, Device):
 class GenericCamera(Device):
 
 	family = StringProperty('Camera')
@@ -26,6 +28,9 @@ class GenericCamera(Device):
 	def get_pixel_height(self):
 		return None
 
+	def get_sensor_temperature(self):
+		return None
+
 	def get_image(self):
 		if hasattr(self, 'last_capture'):
 			return self.last_capture
