@@ -12,16 +12,19 @@ from kivy.uix.boxlayout import BoxLayout
 from kivy.uix.gridlayout import GridLayout
 
 from jocular.component import Component
-from jocular.widgets import Panel, TextInputC, JMDToggleButton
+from jocular.widgets.widgets import TextInputC, JMDToggleButton
+from jocular.panel import Panel
+
 
 def exp_to_str(e):
     if e < .001:
-        return '{:.0f} µs'.format(e*1e6)
+        return f'{e*1e6:.0f} µs'
     if e < 1:
-        return '{:.0f} ms'.format(e*1000)
+        return f'{e*1000:.0f} ms'
     if e - int(e) < .0001:
-        return '{:.0f}s'.format(e)
-    return '{:.2f}s'.format(e)
+        return f'{e:.0f}s'
+    return f'{e:.2f}s'
+
 
 def str_to_exp(s):
     s = str(s)
@@ -42,6 +45,7 @@ def str_to_exp(s):
 
 class ExposureChooser(Panel, Component):
 
+
     def __init__(self, **args):
         super().__init__(**args)
         self.app = App.get_running_app()
@@ -52,16 +56,19 @@ class ExposureChooser(Panel, Component):
             self.user_expos = []
         self.build()
 
+
     def save(self, dt=None):
         with open(self.app.get_path('user_exposures.json'), 'w') as f:
             json.dump(self.user_expos, f, indent=1)       
 
+
     def on_show(self):
         # ensure exposurechooser reflects current state
         e_str = exp_to_str(Component.get('CaptureScript').get_exposure())
         for e, but in self.expo_buttons.items():
             but.state = 'down' if e == e_str else 'normal'
 
+
     def _button(self, name, expo):
         return JMDToggleButton(
                 text=name, 
@@ -71,6 +78,7 @@ class ExposureChooser(Panel, Component):
                 height=dp(24),
                 on_press=partial(self.exposure_selected, expo))
 
+
     def build(self, *args):
 
         self.expos = [
@@ -163,6 +171,7 @@ class ExposureChooser(Panel, Component):
 
         self.app.gui.add_widget(self)
 
+
     @logger.catch()
     def custom_exposure_added(self, tb):
         try:
@@ -178,11 +187,11 @@ class ExposureChooser(Panel, Component):
                 self.user_gl.add_widget(Label(size_hint=(.2, None), height=dp(30)))
             self.user_exposure_box.text = ''
             self.save()
-        except Exception as e:
+        except:
             pass
 
+
     def exposure_selected(self, expo, *args):
         # set exposure on GUI and on scripts panel
         Component.get('CaptureScript').exposure_changed(expo)
-        # self.hide()
 
