@@ -13,20 +13,19 @@ from kivy.lang import Builder
 from kivymd.uix.boxlayout import MDBoxLayout
 
 from jocular.component import Component
-from jocular.settingsmanager import Settings
+from jocular.settingsmanager import JSettings
+from jocular.widgets.widgets import Pin
 
 
 Builder.load_string('''
 
 <MyBoxLayout@BoxLayout>:
     size_hint: (1, None)
-    # height: '{:}dp'.format(int(app.form_font_size[:-2]) + 20)
     height: '42dp'
 
 <SessionInfo>:
     padding: '10dp'
     adaptive_height: True
-    #adaptive_width: True
     pos_hint: {'y': 0, 'x': 0} if root.session.show_session else {'y': 0, 'right': -1000} 
     size_hint: None, None
     orientation: 'vertical'
@@ -39,7 +38,6 @@ Builder.load_string('''
         JTextField:
             width: '180dp'
             height: '32dp'
-            #helper_text: ''
             on_focus: root.session.session_changed(self.text) if not self.focus else None
             text: root.session.session
 
@@ -47,7 +45,6 @@ Builder.load_string('''
         JTextField:
             hint_text: 'transparency'
             width: '220dp'
-            #helper_text: 'e.g. high clouds, good'
             on_focus: root.session.transparency_changed(self.text) if not self.focus else None
             text: root.session.transparency
 
@@ -56,21 +53,18 @@ Builder.load_string('''
         JTextField:
             hint_text: 'seeing'
             width: '240dp'
-            #helper_text: 'e.g. poor, excellent'
             on_focus: root.session.seeing_changed(self.text) if not self.focus else None
             text: root.session.seeing
 
     MyBoxLayout:
         JTextField:
             hint_text: 'temperature'
-            #helper_text: 'e.g. 5C or 45F'
             on_focus: root.session.temperature_changed(self.text) if not self.focus else None
             text: root.session.formatted_temperature
 
         JTextField:
             width: '140dp'
             hint_text: 'brightness'
-            #helper_text: 'e.g. 19.23 or 4.5'
             on_focus: root.session.sky_brightness_changed(self.text) if not self.focus else None
             text: root.session.formatted_sky_brightness
 
@@ -102,32 +96,42 @@ Builder.load_string('''
             text: root.session.session_notes
             on_focus: root.session.session_notes_changed(self.text) if not self.focus else None
 
+    BoxLayout:
+        size_hint: (1, None)
+        height: '22dp'
+
 ''')
 
 
 date_time_format = '%d %b %y %H:%M'
 
+
 def datenow():
     return datetime.now().strftime(date_time_format)
 
+
 def hours_since_date(mydate):
     return (datetime.now() - datetime.strptime(mydate, date_time_format)).total_seconds()/3600
 
+
 # from http://unihedron.com/projects/darksky/NELM2BCalc.html
 def SQM_to_NELM(sqm):
     return 7.93 - 5 * math.log(10 ** (4.316-(sqm / 5)) + 1)
 
+
 def NELM_to_SQM(nelm):
     return 21.58 - 5 * math.log(10 ** (1.586 - nelm / 5) - 1)
 
 
+
 class SessionInfo(MDBoxLayout):
     def __init__(self, session, **kwargs):
         self.session = session
         super().__init__(**kwargs)
 
 
-class Session(Component, Settings):
+
+class Session(Component, JSettings):
 
     save_settings = ['show_session']
 
@@ -167,27 +171,38 @@ class Session(Component, Settings):
     props = ['sky_brightness', 'temperature', 'seeing', 'transparency', 'session_notes', 
         'telescope', 'camera', 'session']
 
+
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
         self.app = App.get_running_app()
+        show_session = self.app.gui.get_gui_setting('show_session')
+        self.show_session = False if show_session is None else show_session
         self.session_info = SessionInfo(self)
         self.app.gui.add_widget(self.session_info)
+        self.app.gui.add_widget(Pin(
+            comp=self, 
+            field='show_session', 
+            loc='lower-left', 
+            tooltip_text='toggle session information panel',
+            show_text='Session'))
+
 
     def describe(self):
         ''' return information for Snapshotter
         '''
-
         return [
             self.session, 
             self.formatted_sky_brightness,
             self.formatted_temperature,
-            'seeing {:}'.format(self.seeing) if self.seeing else None,
-            'transp {:}'.format(self.transparency) if self.transparency else None
+            f'seeing {self.seeing}' if self.seeing else None,
+            f'transp {self.transparency}' if self.transparency else None
             ]
 
+
     def theme_changed(self, *args):
         self.session_info.theme_changed()
 
+
     def temperature_changed(self, temp):
         ''' called when temperature field is altered; parse various formats
             with and without degrees and C or F
@@ -210,6 +225,7 @@ class Session(Component, Settings):
         self.on_temperature()
         self.check_for_change() 
 
+
     def get_temperature(self):
         ''' return string temperature in C or F, or empty string if error
             in conversion or non-existent temperature
@@ -219,8 +235,9 @@ class Session(Component, Settings):
         except:
             return ''
         if self.temperature_units == 'Centigrade':
-            return '{:.0f}\N{DEGREE SIGN}C'.format(temp)
-        return '{:.0f}\N{DEGREE SIGN}F'.format(temp*1.8 + 32)
+            return f'{temp:.0f}\N{DEGREE SIGN}C'
+        return f'{temp*1.8 + 32:.0f}\N{DEGREE SIGN}F'
+
 
     @logger.catch()
     def on_temperature(self, *args):
@@ -230,12 +247,14 @@ class Session(Component, Settings):
         self.formatted_temperature = 'xyz'  # forces an update
         self.formatted_temperature = self.get_temperature()
 
+
     def on_temperature_units(self, *args):
         ''' whenever user changes temperature preferences in config screen,
             ensure it is applied immediately
         '''
         self.formatted_temperature = self.get_temperature()
 
+
     ''' similar methods for sky brightness
     '''
 
@@ -253,27 +272,31 @@ class Session(Component, Settings):
                     bright = bright[:-3].strip()
                 bright = float(bright)
                 self.sky_brightness = bright if units == 'SQM' else SQM_to_NELM(bright)
-        except Exception as e:
+        except:
             self.sky_brightness = None
         self.on_sky_brightness()
         self.check_for_change() 
 
+
     def get_sky_brightness(self):
         try:
             bright = float(self.sky_brightness)
         except:
             return ''
         if self.sky_brightness_units == 'SQM':
-            return '{:.2f} sqm'.format(bright)
-        return '{:.1f} nelm'.format(NELM_to_SQM(bright))
+            return f'{bright:.2f} sqm'
+        return f'{NELM_to_SQM(bright):.1f} nelm'
+
 
     def on_sky_brightness(self, *args):
         self.formatted_sky_brightness = 'xyz'
         self.formatted_sky_brightness = self.get_sky_brightness()
 
+
     def on_sky_brightness_units(self, *args):
         self.formatted_sky_brightness = self.get_sky_brightness()
 
+
     ''' remaining changes
     '''  
 
@@ -281,14 +304,17 @@ class Session(Component, Settings):
         self.session_notes = session_notes
         self.check_for_change() 
 
+
     def seeing_changed(self, seeing):
         self.seeing = seeing
         self.check_for_change() 
 
+
     def transparency_changed(self, transparency):
         self.transparency = transparency
         self.check_for_change() 
 
+
     def telescope_changed(self, telescope):
         ''' Since we persist this between sessions, ensure we save
             the session on any change; likewise with camera
@@ -297,15 +323,18 @@ class Session(Component, Settings):
         self.check_for_change()
         self.save_session()
 
+
     def camera_changed(self, camera):
         self.camera = camera
         self.check_for_change()
         self.save_session() 
 
+
     def session_changed(self, session):
         self.session = session
         self.check_for_change() 
 
+
     @logger.catch()
     def check_for_change(self):
         ''' Check if any property has changed and stack is not empty
@@ -330,6 +359,7 @@ class Session(Component, Settings):
         self.load_session()
         self.initial_values = {p: getattr(self, p) for p in self.props}
 
+
     @logger.catch()
     def on_previous_object(self):
         ''' apply previous session settings, keeping a record so changes can be monitored
@@ -340,10 +370,12 @@ class Session(Component, Settings):
         self.apply_session_settings(settings)
         self.initial_values = {p: getattr(self, p) for p in self.props}
 
+
     def cancel_session_clock(self):
         if hasattr(self, 'session_clock'):
             self.session_clock.cancel()
 
+
     def load_session(self):
         ''' Try to get information from last session and apply it, otherwise
             create a new session
@@ -360,13 +392,15 @@ class Session(Component, Settings):
                 self.camera = session.get('camera', '')
         except Exception as e:
             # any problems we just start a fresh session
-            logger.debug('problem loading session: recreating {:}'.format(e))
+            logger.debug(f'problem loading session: recreating {e}')
             self.create_empty_session()
+            self.save_session()
 
         # create date and clock to update it
         self.session = datetime.now().strftime(date_time_format)
         self.session_clock = Clock.schedule_interval(self.update_time, 1.0)
 
+
     def create_empty_session(self):
         ''' set all properties except session to empty/None
         '''
@@ -375,6 +409,7 @@ class Session(Component, Settings):
         for p in set(self.props) - {'session', 'temperature', 'sky_brightness'}:
             setattr(self, p, '')
 
+
     def get_session(self):
         # accessor method that strips off seconds if present
         if self.session.count(':') == 2:
@@ -382,6 +417,7 @@ class Session(Component, Settings):
         else:
             return self.session
 
+
     def current_session_info(self):
         session = {}
         for p in self.props:
@@ -391,6 +427,7 @@ class Session(Component, Settings):
         session['session'] = self.get_session()
         return session
 
+
     def apply_session_settings(self, settings):
         ''' apply 'settings', by clearing existing settings and
             overriding
@@ -406,6 +443,7 @@ class Session(Component, Settings):
         with open(self.app.get_path('last_session.json'), 'w') as f:
             json.dump(self.current_session_info(), f, indent=1)    
 
+
     @logger.catch()
     def on_save_object(self):
 
